'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = initWeb3;

var _web = require('web3');

var _web2 = _interopRequireDefault(_web);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * @function initWeb3
 *
 * @description Initialises web3 object globally
 *
 * @param {object} payload - Payload object
 * @param {object} payload.props - API function properties
 * @param {string} payload.props.rpcaddr - RPC address of Ethereum node to connect on
 * @param {number} payload.props.rpcport - RPC port of Ethereum node to connect on
 * @param {boolean} [payload.props.ssl] - Defines using of ssl for connection or not
 *
 * @returns {object} The same (as input) payload object
 */
function initWeb3(payload) {
  var rpcEndpoint = getRPCEndpoint(payload.props);

  // check if web3 object already injected in global scope
  if (isWeb3Injected(rpcEndpoint)) {
    return payload;
  }

  var web3 = new _web2.default(new _web2.default.providers.HttpProvider(rpcEndpoint));

  setGlobalWeb3(web3, rpcEndpoint);

  return payload;
} /**
   * @file Manages helper functions to init web3 object globally
   * @author Ivan Violentov <ivan.violentov@jibrel.network>
   */

function isWeb3Injected(rpcEndpoint) {
  var globalScope = typeof window !== 'undefined' ? window : global;

  if (globalScope.web3Endpoint !== rpcEndpoint) {
    return false;
  }

  return checkWeb3IsConnected(globalScope.web3);
}

function getRPCEndpoint(_ref) {
  var rpcaddr = _ref.rpcaddr,
      rpcport = _ref.rpcport,
      ssl = _ref.ssl;

  return 'http' + (ssl ? 's' : '') + '://' + rpcaddr + ':' + rpcport;
}

function setGlobalWeb3(web3, rpcEndpoint) {
  var globalScope = typeof window !== 'undefined' ? window : global;

  if (!checkWeb3IsConnected(web3)) {
    throw new Error('Could not set web3 as global object because it is not connected to the node');
  }

  globalScope.web3 = web3;
  globalScope.web3Endpoint = rpcEndpoint;
}

function checkWeb3IsConnected(web3) {
  var isWeb3Connected = web3 && web3.isConnected();

  if (!isWeb3Connected) {
    return false;
  }

  return checkWeb3ethSupportedMethods(web3);
}

function checkWeb3ethSupportedMethods(web3) {
  var web3eth = web3.eth;

  if (!web3eth) {
    throw new Error('web3.eth is not supported');
  }

  var supportedMethods = ['getBalance', 'getBlock', 'contract', 'filter', 'getTransaction', 'getTransactionCount', 'getTransactionReceipt', 'getGasPrice', 'estimateGas', 'sendRawTransaction'];

  supportedMethods.forEach(function (method) {
    if (!(method in web3eth)) {
      throw new Error('web3.eth.' + method + ' is not supported');
    }
  });

  return true;
}